#!/usr/bin/perl -w 

#
# $Id: pseudonyms.pm,v 1.20 2005/04/26 15:39:42 charlieb Exp $
#

package esmith::FormMagick::Panel::mailalias;

use strict;
use esmith::AccountsDB;
use esmith::DomainsDB;
use esmith::ConfigDB;
use esmith::FormMagick;
use esmith::util;
use File::Basename;
use Exporter;
use Carp;
use URI::Escape;
use esmith::DB

our @ISA = qw(esmith::FormMagick Exporter);

our @EXPORT = qw( 
    get_prop 
    get_value 
    get_cgi_param
    performCreateMailalias  
    performModifyMailalias 
    performRemoveMailalias 
    existing_accounts 
    existing_domains
    print_begin_page 
    print_hidden_mailalias_field
    get_mailalias_domain
    get_mailalias_email 
    is_mailalias_internal
    is_mailalias_not_removable
    validate_new_mailalias_name
    validate_is_mailalias 
    validate_new_email
);

our $VERSION = sprintf '%d.%03d', q$Revision: 1.20 $ =~ /: (\d+).(\d+)/;

our $config = esmith::ConfigDB->open();
our $accounts = esmith::AccountsDB->open();
our $domains = esmith::DomainsDB->open();


# {{{ header

=pod 

=head1 NAME

esmith::FormMagick::Panels::pseudonyms - useful panel functions

=head1 SYNOPSIS

    use esmith::FormMagick::Panels::pseudonyms;

    my $panel = esmith::FormMagick::Panel::pseudonyms->new();
    $panel->display();

=head1 DESCRIPTION

=cut

# }}}

# {{{ new

=head2 new();

Exactly as for esmith::FormMagick

=begin testing


use_ok('esmith::FormMagick::Panel::pseudonyms');
use vars qw($panel);
ok($panel = esmith::FormMagick::Panel::pseudonyms->new(), "Create panel object");
isa_ok($panel, 'esmith::FormMagick::Panel::pseudonyms');

=end testing

=cut

sub new {
    shift;
    my $self = esmith::FormMagick->new();
    $self->{calling_package} = (caller)[0];
    bless $self;
    return $self;
}

# }}}

=head2 get_cgi_param FM FIELD

Returns the named CGI parameter as a string

=cut

sub get_cgi_param {
    my $fm = shift;
    my $param = shift;

    return ($fm->{'cgi'}->param($param));
}

# {{{ get_prop

=head2 get_prop ITEM PROP

A simple accessor for esmith::ConfigDB::Record::prop

=cut

sub get_prop {
    my $fm = shift;
    my $item = shift;
    my $prop = shift;

    my $record = $config->get($item);
    if ($record) {
        return $record->prop($prop);
    }
    else {
        return '';
    }
}

# }}}

# {{{ get_value 

=head2 get_value ITEM

A simple accessor for esmith::ConfigDB::Record::value

=cut

sub get_value {
    my $fm = shift;
    my $item = shift;

    my $record = $config->get($item);
    if ($record) {
        return $record->value();
    }
    else {
        return '';
    }
}

# }}}

=head1 ACTION

=head2 performCreatePseudonym

Create a new pseudonym

=cut

# {{{ performCreatePseudonym

sub performCreateMailalias  {
    my $fm = shift;
    my $q = $fm->{'cgi'};
    my $email = uri_unescape($q->param ('email'));
    my $domain = uri_unescape($q->param ('domain'));
    my $mailalias = uri_unescape($q->param('mailalias')); 
    my $msg = "OK";

    $accounts->new_record($mailalias, { type => 'mailalias',
					domain => $domain, 
                                     email => $email} )
        or $msg = "Error occurred while creating mailalias in database.";

    # Untaint $pseudonym before use in system()
    ($mailalias) = ($mailalias =~ /([\w\p{L}.]+)/ );
    system( "/sbin/e-smith/signal-event", "mailalias-create", "$mailalias",) 
        == 0 or $msg = "Error occurred while creating mailalias.";

    if ($msg eq "OK") 
    {
        $q->delete('email');
        $q->delete('mailalias');
	$q->delete('domain');
        $fm->success('CREATE_SUCCEEDED');
    }
    else
    {
        $fm->error($msg);
    }
}
# }}}

# {{{ performModifyPseudonym

=head2 performModifyPseudonmy

Modify a pseudonym.

=cut

sub performModifyMailalias {
    my $fm = shift;
    my $q = $fm->{'cgi'};
    my $msg = "OK";

    my $mailalias = uri_unescape($q->param ('mailalias'));
    my $domain = uri_unescape($q->param ('domain'));
    my $email = uri_unescape($q->param ('email'));
    my $internal = $q->param ('internal') || 'NO';
    my $removable = $accounts->get($mailalias)->prop('Removable') || 'yes';

    my @props = ('domain', $domain , 'email',"$email");

    if ($removable eq 'yes') {
        if ($internal eq "YES") {$accounts->get($mailalias)->set_prop('Visible','internal'); }
        else { $accounts->get($mailalias)->delete_prop('Visible'); }
    }

    $accounts->get($mailalias)->set_prop('email',"$email")
        or $msg = "Error occurred while modifying mailalias in database.";
    $accounts->get($mailalias)->set_prop('domain', $domain)
        or $msg = "Error occurred while modifying mailalias in database.";
	
    # Untaint $mailalias before use in system()
    ($mailalias) = ($mailalias =~ /([\w\p{L}]+.)/);
    system( "/sbin/e-smith/signal-event", "mailalias-modify", "$mailalias",)
        == 0 or $msg = "Error occurred while modifying mailalias.";

    if ($msg eq "OK")
    {
        $q->delete('account');
        $q->delete('mailalias');
        $q->delete('internal');
        $fm->success('MODIFY_SUCCEEDED');
    }
    else
    {
        $fm->error($msg);
    }
}
# }}}

# {{{ performRemoveMailalias

sub performRemoveMailalias {
    my $fm = shift;
    my $q = $fm->{'cgi'};
    my $msg = "OK";

    my $mailalias = uri_unescape($q->param('mailalias'));

    unless($fm->validate_is_mailalias($mailalias) eq 'OK') {
        $fm->{cgi}->param( -name => 'wherenext', -value => 'InvalidMailalias' );
        return '';
    }

    #------------------------------------------------------------
    # Make the mailalias inactive, signal mailalias-delete event
    # and then delete it
    #------------------------------------------------------------

    # Untaint $mailalias before use in system()
    ($mailalias) = ($mailalias =~ /([\w\p{L}]+.)/);
    system( "/sbin/e-smith/signal-event", "mailalias-delete", "$mailalias",)
        == 0 or $msg = "Error occurred while deleting mailalias.";

    $accounts->get($mailalias)->delete()
        or $msg = "Error occurred while deleting mailalias from database.";

    if ($msg eq "OK")
    {
        $q->delete('mailalias');
        $fm->success('REMOVE_SUCCEEDED');
    }
    else
    {
        $fm->error($msg);
    }

    return '';
}

# }}} remove pseudonym

# {{{ existing_accounts

=head2 existing_accounts

Return a hash of exisitng system accounts

=cut

sub existing_accounts {
    my $fm = shift;
    my %existingAccounts = ('admin' => "Administrator" );

    foreach my $account ($accounts->get_all) {
        if ($account->prop('type') =~ /(user|group)/) {
            $existingAccounts{$account->key} = $account->key;
        }
        if ($account->prop('type') eq "pseudonym") {
            my $target = $accounts->get($account->prop('Account'));

            unless ($target)
            {
                warn "WARNING: pseudonym (" 
			. $account->key 
			. ") => missing Account(" 
			. $account->prop('Account') 
			. ")\n";
                next;
            }

            $existingAccounts{$account->key} = $account->key 
                unless ($target->prop('type') eq "pseudonym");
        }
    }
    return(\%existingAccounts); 
}

# }}}

sub existing_domains {
    my $fm = shift;
    my %existingDomains = ('ALL' => "ALL" );

    foreach my $domain ($domains->get_all) {
        if ($domain->prop('type') =~ /(domain)/) {
            $existingDomains{$domain->key} = $domain->key;
        }
    }
    return(\%existingDomains); 
}

# {{{ print_begin_page

=head2 print_begin_page 

Print the initial page of the ofrm

=cut

sub print_begin_page {
    my $fm = shift;
    my $q = $fm->{'cgi'};
    my @emailMailaliases;
    # Need to untie and re-tie the accounts database to ensure changes
    # are recognised.
    foreach my $account ($accounts->get_all()) {
        my $type = $account->prop('type');
        my $key = $account->key();
        push (@emailMailaliases, $key) if ($type eq 'mailalias');
    }
    my $urlprefix = $fm->build_cgi_params(uri_unescape($q->param('mailalias')));    
    print "  <tr>\n    <td colspan='2'>\n";
    print $q->p($q->a({href => "mailalias?$urlprefix&wherenext=Create", -class => "button-like"},
                $fm->localise("CLICK_TO_CREATE")));

    my $numMailaliases = @emailMailaliases;
    if ($numMailaliases == 0) {
        print $q->h2($fm->localise('NO_MAILALIASES'));
    } else {
        print $q->h2($fm->localise('CURRENT_MAILALIASES'));
        print $q->start_table ({-CLASS => "sme-border"});
        print "<tr>      <th class=\"sme-border\">".$fm->localise('MAILALIAS') ."</th>\n".
            "      <th class=\"sme-border\">".$fm->localise('DOMAINNAME') . "</th>\n" .
            "      <th class=\"sme-border\">".$fm->localise('EMAIL_DEST') . "</th>\n" .	    	    
            "      <th class=\"sme-border\" colspan=\"2\">".$fm->localise('ACTION') . "</th></tr>\n";

        foreach my $mailalias (sort @emailMailaliases) {
            #my $account = $accounts->get($mailalias)->prop('Account');
	    my $email = $accounts->get($mailalias)->prop('email');
            my $domain = $accounts->get($mailalias)->prop('domain');
	    	    
	    #$account = "Administrator" if ($account eq "admin");
            $domain = $fm->localise("ALL") if ($domain eq "ALL");

            my $removable = $accounts->get($mailalias)->prop('Removable') || 'yes';
            my $changeable = $accounts->get($mailalias)->prop('Changeable') || 'yes';

            my $visible = $accounts->get($mailalias)->prop('Visible');
            #$account .= $fm->localise("LOCAL_ONLY")
            #    if (defined $visible && $visible eq "internal");

            my $urlprefix = $fm->build_cgi_params($mailalias);
	    $email=~s/,/<br>/; # substitution for printing
            print "    <tr>\n" .
                "      <td class=\"sme-border\">$mailalias</td>\n" .
                "      <td class=\"sme-border\">$domain</td>\n" .		
                "      <td class=\"sme-border\">$email</td>\n";
            if ($changeable eq 'no') {
                print "      <td class=\"sme-border\">&nbsp;</td>\n";
            } else {
                print "      <td class=\"sme-border\">".$q->a({href => "mailalias?$urlprefix&wherenext=Modify"},$fm->localise("MODIFY"))."</td>\n";
            }
            if ($removable eq 'no') {
                print "      <td class=\"sme-border\">&nbsp;</td>\n";
            } else {
                print "      <td class=\"sme-border\">".$q->a({href => "mailalias?$urlprefix&wherenext=Remove"},$fm->localise("REMOVE"))."</td>\n";
            }
	    
            print "</tr>";
        }

        print $q->end_table,"\n";
    }
    print "</td></tr>";
    return '';
}

# }}}

# {{{ get_mailalias_domain

=head2 get_mailalias_domain

Returns the current domain property for this mailalias

=cut

sub get_mailalias_domain {
    my $fm = shift;
    my $q = $fm->{'cgi'};
    my $mailalias = uri_unescape($q->param('mailalias'));
    my $domain = $accounts->get($mailalias)->prop('domain');
    if ($domain eq "") {
        $domain = "ALL";
    } 
    return($domain);
}
# }}}


# {{{ get_mailalias_email

=head2 get_mailalias_email

Returns the current email property for this mailalias

=cut

sub get_mailalias_email {
    my $fm = shift;
    my $q = $fm->{'cgi'};
    my $mailalias = uri_unescape($q->param('mailalias'));
    my $email = $accounts->get($mailalias)->prop('email');
   
    return($email);
}
# }}}



# {{{ is_mailalias_not_removable

=head2 is_mailalias_not_removable

Returns 1 if the current Account is not removable, 0 otherwise

=cut

sub is_mailalias_not_removable {
    my $fm = shift;
    my $q = $fm->{'cgi'};
    my $mailalias = uri_unescape($q->param('mailalias'));
    my $removable = $accounts->get($mailalias)->prop('Removable') || 'yes';
    return 1  if ($removable eq 'yes');
    return 0;
}
# }}}

# {{{ is_mailalias_internal

=head2 is_mailalias_internal

Returns YES if the current Account property Visible is 'internal'

=cut

sub is_mailalias_internal {
    my $fm = shift;
    my $q = $fm->{'cgi'};
    my $mailalias = uri_unescape($q->param('mailalias'));
    my $visible = $accounts->get($mailalias)->prop('Visible') || '';
    return 'YES'  if ($visible eq 'internal');
    return 'NO';
}
# }}}

# {{{ validate_new_pseudonym_name 

=head2 validate_new_pseudonym_name FM PSEUDONYM

Returns "OK" if the pseudonym starts with a letter or number and
contains only letters, numbers, . - and _ and isn't taken

Returns  "VALID_PSEUDONYM_NAMES"  if the name contains invalid chars

Returns "NAME_IN_USE" if this pseudonym is taken.

=begin testing

ok(esmith::FormMagick::Panel::pseudonyms::validate_new_pseudonym_name('',"23skidoo") eq 'OK', "23skidoo is a valid pseudonym");
ok(esmith::FormMagick::Panel::pseudonyms::validate_new_pseudonym_name('',"_23skidoo") ne 'OK', "_23skidoo is not  a valid pseudonym");
ok(esmith::FormMagick::Panel::pseudonyms::validate_new_pseudonym_name('',"_23skidoo") eq 'VALID_PSEUDONYM_NAMES', "_23skidoo is  an invalid pseudonym");

=end testing


=cut

sub validate_new_mailalias_name {
    my $fm = shift;
    my $mailalias = shift;

    my $acct = $accounts->get($mailalias);
    if (defined $acct) {
        return('NAME_IN_USE');
    }
    elsif ($mailalias =~ /@/)
    {
 
        return ('INVALID_MAILALIAS_NAMES');
    }
    elsif ($mailalias !~ /^([a-z0-9][a-z0-9\.\-_!#\?~\$\^\+&`%\/\*]*)$/)
    {
        return('VALID_MAILALIAS_NAMES');
    }
    else {
        return('OK');
    }
}

# }}}

# {{{ validate_is_mailalias 

=head2 validate_is_mailalias FM NAME

returns  "OK" if it is.
returns "NOT_A_MAILALIAS" if the name in question isn't an existing mailalias

=cut

sub validate_is_mailalias {
    my $fm = shift;
    my $mailalias = shift;

    $mailalias = $accounts->get($mailalias);

    return('NOT_A_MAILALIAS') unless $mailalias;
    my $type = $mailalias->prop('type');

    unless (defined $type && ($type eq 'mailalias') ) {

        return('NOT_A_MAILALIAS');
    }
    return ('OK');
}

# }}}
                                                                 
=head2 emailforward()

Validation routine for email forwarding

=cut

sub validate_new_email {
    my $fm = shift;
    my $email= shift;
    
    if ($email eq "")
    {
  
        return $fm->localise('CANNOT_CONTAIN_WHITESPACE');
    }
    else
    {
    
        return $fm->localise('CANNOT_CONTAIN_WHITESPACE')
            if ( $email =~ /\s+/ );
	    
        # Permit a local address, an email or multiple email separate by a coma.
	my @mail = split(/,/,$email.",");
	foreach (@mail )
	{
        return "OK" if $_ =~ /^[a-zA-Z][a-zA-Z0-9\._\-@,]*$/;
        return $fm->localise('UNACCEPTABLE_CHARS');
	}
    }
}

=head2 build_cgi_params()                                        
                                                                 
Builds a CGI query string, using various sensible                
defaults and esmith::FormMagick's props_to_query_string() method.
                                                                 
=cut                                                             
                                                                 
sub build_cgi_params {                                           
    my ($fm, $mailalias) = @_;                                       

    my %props = (                                                
            page       => 0,                                            
            page_stack => "",                                        
            ".id"      => $fm->{cgi}->param('.id') || "",         
            mailalias  => $mailalias,                                   
            );                                                           

    return $fm->props_to_query_string(\%props);                  
}                                                                

=head2 print_hidden_mailalias_field FM

prints a hidden form field containing the current value of the mailalias
attribute


=cut

sub print_hidden_mailalias_field {
    my $fm = shift;
    print "<input type='hidden' name='mailalias' value=\"".uri_escape($fm->{'cgi'}->param('mailalias'))."\">";
    return '';
}

1;
